import datetime

from functions import create_appointment, create_location, create_worker, create_appointment_type
from functions import create_subject
from functions import create_visit
from web.models import Appointment, Location
from web.views.notifications import \
    get_approaching_visits_for_mail_contact, \
    get_approaching_visits_for_mail_contact_count, \
    get_approaching_visits_without_appointments, \
    get_approaching_visits_without_appointments_count, \
    get_exceeded_visit_notifications_count, \
    get_notifications, \
    get_subject_with_no_visit_notifications_count, \
    get_subjects_with_reminder_count, \
    get_visits_without_appointments_count, \
    get_visits_with_missing_appointments_count, \
    get_today_midnight_date, \
    get_unfinished_appointments, \
    get_unfinished_appointments_count, \
    get_unfinished_visits
from . import LoggedInTestCase


class NotificationViewTests(LoggedInTestCase):
    def test_get_exceeded_visit_notifications_count(self):
        original_notification = get_visits_without_appointments_count(self.user)

        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_end = "2011-01-01"
        visit.save()
        create_appointment(visit)

        notification = get_exceeded_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_exceeded_visit_notifications_count_2(self):
        original_notification = get_visits_without_appointments_count(self.user)

        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_end = "2011-01-01"
        visit.is_finished = True
        visit.save()
        create_appointment(visit)

        notification = get_exceeded_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_visits_without_appointments_count(self):
        original_notification = get_visits_without_appointments_count(self.user)
        subject = create_subject()
        create_visit(subject)

        notification = get_visits_without_appointments_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_visits_with_missing_appointments_count(self):
        original_notification = get_visits_with_missing_appointments_count(self.user)

        appointment_type = create_appointment_type()
        visit = create_visit()
        visit.appointment_types.add(appointment_type)
        visit.save()

        notification = get_visits_with_missing_appointments_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_visits_with_missing_appointments_count_2(self):
        original_notification = get_visits_with_missing_appointments_count(self.user)

        appointment_type = create_appointment_type()
        visit = create_visit()
        visit.appointment_types.add(appointment_type)
        visit.save()

        appointment = create_appointment(visit)
        appointment.appointment_types.add(appointment_type)
        appointment.status = Appointment.APPOINTMENT_STATUS_FINISHED
        appointment.save()

        notification = get_visits_with_missing_appointments_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_notifications(self):
        create_worker(self.user)
        result = get_notifications(self.user)

        self.assertEquals(0, result[0])
        self.assertTrue(isinstance(result[1], list))
        self.assertTrue(len(result[1]) > 0)

    def test_get_visits_without_appointments_count_2(self):
        appointment_type = create_appointment_type()
        original_notification = get_visits_without_appointments_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.appointment_types.add(appointment_type)
        visit.save()

        notification = get_visits_without_appointments_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_visits_without_appointments_count_3(self):
        original_notification = get_visits_without_appointments_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        appointment = create_appointment(visit)

        appointment.status = Appointment.APPOINTMENT_STATUS_CANCELLED
        appointment.save()

        notification = get_visits_without_appointments_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_unfinished_visits_order(self):
        subject = create_subject()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=-10)
        visit.save()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=-8)
        visit.save()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=-12)
        visit.save()

        visits = get_unfinished_visits(self.user)
        self.assertEquals(3, len(visits))

        # check sort order
        self.assertTrue(visits[0].datetime_begin < visits[1].datetime_begin)
        self.assertTrue(visits[1].datetime_begin < visits[2].datetime_begin)

    def test_get_approaching_visits_without_appointments_count(self):
        original_notification = get_approaching_visits_without_appointments_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=2)
        visit.save()

        notification = get_approaching_visits_without_appointments_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_approaching_visits_without_appointments_count_2(self):
        original_notification = get_approaching_visits_without_appointments_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=2)
        visit.save()
        create_appointment(visit)

        notification = get_approaching_visits_without_appointments_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_approaching_visits_without_appointments_count_3(self):
        original_notification = get_approaching_visits_without_appointments_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=2)
        visit.save()
        appointment = create_appointment(visit)

        appointment.status = Appointment.APPOINTMENT_STATUS_CANCELLED
        appointment.save()

        notification = get_approaching_visits_without_appointments_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_approaching_visits_without_appointments_order(self):
        subject = create_subject()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=10)
        visit.save()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=8)
        visit.save()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=12)
        visit.save()

        visits = get_approaching_visits_without_appointments(self.user)
        self.assertEquals(3, visits.count())

        # check sort order
        self.assertTrue(visits[0].datetime_begin < visits[1].datetime_begin)
        self.assertTrue(visits[1].datetime_begin < visits[2].datetime_begin)

    def test_get_subject_with_no_visit_notifications_count(self):
        original_notification = get_subject_with_no_visit_notifications_count(self.user)
        create_subject()

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_subject_with_no_visit_notifications_count_2(self):
        original_notification = get_subject_with_no_visit_notifications_count(self.user)
        subject = create_subject()

        visit = create_visit(subject)
        visit.is_finished = True
        visit.save()

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_subject_with_no_visit_notifications_count_3(self):
        original_notification = get_subject_with_no_visit_notifications_count(self.user)
        subject = create_subject()

        create_visit(subject)

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_subject_with_no_visit_notifications_count_4(self):
        original_notification = get_subject_with_no_visit_notifications_count(self.user)
        subject = create_subject()
        subject.dead = True
        subject.save()

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_subject_with_no_visit_notifications_count_5(self):
        original_notification = get_subject_with_no_visit_notifications_count(self.user)
        subject = create_subject()
        subject.resigned = True
        subject.save()

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_unfinished_appointments_count(self):
        original_notification = get_unfinished_appointments_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        appointment = create_appointment(visit)
        appointment.datetime_when = "2011-01-01"
        appointment.status = Appointment.APPOINTMENT_STATUS_SCHEDULED
        appointment.save()

        notification = get_unfinished_appointments_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_unfinished_appointments_count_2(self):
        original_notification = get_unfinished_appointments_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        appointment = create_appointment(visit)
        appointment.datetime_when = "2011-01-01"
        appointment.status = Appointment.APPOINTMENT_STATUS_CANCELLED
        appointment.save()

        notification = get_unfinished_appointments_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_unfinished_appointments_order(self):
        subject = create_subject()
        visit = create_visit(subject)

        appointment = create_appointment(visit, get_today_midnight_date() + datetime.timedelta(days=-10))
        appointment.status = Appointment.APPOINTMENT_STATUS_SCHEDULED
        appointment.save()
        appointment = create_appointment(visit, get_today_midnight_date() + datetime.timedelta(days=-8))
        appointment.status = Appointment.APPOINTMENT_STATUS_SCHEDULED
        appointment.save()
        appointment = create_appointment(visit, get_today_midnight_date() + datetime.timedelta(days=-12))
        appointment.status = Appointment.APPOINTMENT_STATUS_SCHEDULED
        appointment.save()

        appointments = get_unfinished_appointments(self.user)
        self.assertEquals(3, appointments.count())

        # check sort order
        self.assertTrue(appointments[0].datetime_when < appointments[1].datetime_when)
        self.assertTrue(appointments[1].datetime_when < appointments[2].datetime_when)

    def test_get_subject_with_no_visit_notifications_count_for_many_locations(self):
        create_location("l1")
        create_location("l2")
        original_notification = get_subject_with_no_visit_notifications_count(self.user)
        create_subject()

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_subject_with_no_visit_notifications_count_for_invalid_location(self):
        worker = create_worker()
        worker.locations = [create_location("l2")]
        worker.save()

        original_notification = get_subject_with_no_visit_notifications_count(worker)

        subject = create_subject()
        subject.default_location = create_location("l1")
        subject.save()

        notification = get_subject_with_no_visit_notifications_count(worker)
        self.assertEquals(original_notification.count, notification.count)

        worker.locations = Location.objects.filter(name="l1")
        worker.save()

        notification = get_subject_with_no_visit_notifications_count(worker)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_approaching_visits_for_mail_contact_count(self):
        original_notification = get_approaching_visits_for_mail_contact_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=100)
        visit.save()

        notification = get_approaching_visits_for_mail_contact_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_approaching_visits_for_mail_contact_count_2(self):
        original_notification = get_approaching_visits_for_mail_contact_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=100)
        visit.save()
        create_appointment(visit)

        notification = get_approaching_visits_for_mail_contact_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_approaching_visits_for_mail_contact_count_3(self):
        original_notification = get_approaching_visits_for_mail_contact_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=100)
        visit.save()
        appointment = create_appointment(visit)

        appointment.status = Appointment.APPOINTMENT_STATUS_CANCELLED
        appointment.save()

        notification = get_approaching_visits_for_mail_contact_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_approaching_visits_for_mail_contact_count_4(self):
        original_notification = get_approaching_visits_for_mail_contact_count(self.user)
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=100)
        visit.post_mail_sent = True
        visit.save()

        notification = get_approaching_visits_for_mail_contact_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

    def test_get_approaching_visits_for_mail_contact_order(self):
        subject = create_subject()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=110)
        visit.save()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=108)
        visit.save()

        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date() + datetime.timedelta(days=112)
        visit.save()

        visits = get_approaching_visits_for_mail_contact(self.user)
        self.assertEquals(3, visits.count())

        # check sort order
        self.assertTrue(visits[0].datetime_begin < visits[1].datetime_begin)
        self.assertTrue(visits[1].datetime_begin < visits[2].datetime_begin)

    def test_get_subjects_with_reminder_count(self):
        original_without_visit_notification = get_subject_with_no_visit_notifications_count(self.user)
        original_notification = get_subjects_with_reminder_count(self.user)
        subject = create_subject()
        subject.datetime_contact_reminder = get_today_midnight_date() + datetime.timedelta(days=-1)
        subject.save()

        notification = get_subjects_with_reminder_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_without_visit_notification.count, notification.count)

    def test_get_subjects_with_reminder_count_2(self):
        original_without_visit_notification = get_subject_with_no_visit_notifications_count(self.user)
        original_notification = get_subjects_with_reminder_count(self.user)
        subject = create_subject()
        subject.datetime_contact_reminder = get_today_midnight_date() + datetime.timedelta(hours=23)
        subject.save()

        notification = get_subjects_with_reminder_count(self.user)
        self.assertEquals(original_notification.count + 1, notification.count)

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_without_visit_notification.count, notification.count)

    def test_get_subjects_with_reminder_count_3(self):
        original_without_visit_notification = get_subject_with_no_visit_notifications_count(self.user)
        original_notification = get_subjects_with_reminder_count(self.user)
        subject = create_subject()
        subject.datetime_contact_reminder = get_today_midnight_date() + datetime.timedelta(days=2)
        subject.save()

        notification = get_subjects_with_reminder_count(self.user)
        self.assertEquals(original_notification.count, notification.count)

        notification = get_subject_with_no_visit_notifications_count(self.user)
        self.assertEquals(original_without_visit_notification.count, notification.count)
